@@ -10,11 +10,11 @@
 macro_rules! hash_to_message {
 	($slice: expr) => {
 		{
-			#[cfg(not(feature = "fuzztarget"))]
+			#[cfg(not(fuzzing))]
 			{
 				::bitcoin::secp256k1::Message::from_slice($slice).unwrap()
 			}
-			#[cfg(feature = "fuzztarget")]
+			#[cfg(fuzzing)]
 			{
 				match ::bitcoin::secp256k1::Message::from_slice($slice) {
 					Ok(msg) => msg,
