@@ -21,8 +21,7 @@ use ln::features::{ChannelFeatures, InitFeatures};
 use ln::msgs;
 use ln::msgs::OptionalField;
 use ln::script::ShutdownScript;
-use routing::scoring::ScorerUsingTime;
-use routing::scoring::time::Eternity;
+use routing::scoring::FixedPenaltyScorer;
 use util::enforcing_trait_impls::{EnforcingSigner, EnforcementState};
 use util::events;
 use util::logger::{Logger, Level, Record};
@@ -47,7 +46,8 @@ use core::time::Duration;
 use sync::{Mutex, Arc};
 use core::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 use core::{cmp, mem};
-use chain::keysinterface::{InMemorySigner, KeyMaterial};
+use bitcoin::bech32::u5;
+use chain::keysinterface::{InMemorySigner, Recipient, KeyMaterial};
 
 pub struct TestVecWriter(pub Vec<u8>);
 impl Writer for TestVecWriter {
@@ -70,7 +70,7 @@ pub struct OnlyReadsKeysInterface {}
 impl keysinterface::KeysInterface for OnlyReadsKeysInterface {
 	type Signer = EnforcingSigner;
 
-	fn get_node_secret(&self) -> SecretKey { unreachable!(); }
+	fn get_node_secret(&self, _recipient: Recipient) -> Result<SecretKey, ()> { unreachable!(); }
 	fn get_inbound_payment_key_material(&self) -> KeyMaterial { unreachable!(); }
 	fn get_destination_script(&self) -> Script { unreachable!(); }
 	fn get_shutdown_scriptpubkey(&self) -> ShutdownScript { unreachable!(); }
@@ -78,7 +78,8 @@ impl keysinterface::KeysInterface for OnlyReadsKeysInterface {
 	fn get_secure_random_bytes(&self) -> [u8; 32] { [0; 32] }
 
 	fn read_chan_signer(&self, mut reader: &[u8]) -> Result<Self::Signer, msgs::DecodeError> {
-		let inner: InMemorySigner = Readable::read(&mut reader)?;
+		let dummy_sk = SecretKey::from_slice(&[42; 32]).unwrap();
+		let inner: InMemorySigner = ReadableArgs::read(&mut reader, dummy_sk)?;
 		let state = Arc::new(Mutex::new(EnforcementState::new()));
 
 		Ok(EnforcingSigner::new_with_revoked(
@@ -87,7 +88,7 @@ impl keysinterface::KeysInterface for OnlyReadsKeysInterface {
 			false
 		))
 	}
-	fn sign_invoice(&self, _invoice_preimage: Vec<u8>) -> Result<RecoverableSignature, ()> { unreachable!(); }
+	fn sign_invoice(&self, _hrp_bytes: &[u8], _invoice_data: &[u5], _recipient: Recipient) -> Result<RecoverableSignature, ()> { unreachable!(); }
 }
 
 pub struct TestChainMonitor<'a> {
@@ -383,7 +384,7 @@ impl msgs::RoutingMessageHandler for TestRoutingMessageHandler {
 		Vec::new()
 	}
 
-	fn sync_routing_table(&self, _their_node_id: &PublicKey, _init_msg: &msgs::Init) {}
+	fn peer_connected(&self, _their_node_id: &PublicKey, _init_msg: &msgs::Init) {}
 
 	fn handle_reply_channel_range(&self, _their_node_id: &PublicKey, _msg: msgs::ReplyChannelRange) -> Result<(), msgs::LightningError> {
 		Ok(())
@@ -410,7 +411,10 @@ impl events::MessageSendEventsProvider for TestRoutingMessageHandler {
 
 pub struct TestLogger {
 	level: Level,
+	#[cfg(feature = "std")]
 	id: String,
+	#[cfg(not(feature = "std"))]
+	_id: String,
 	pub lines: Mutex<HashMap<(String, String), usize>>,
 }
 
@@ -421,7 +425,10 @@ impl TestLogger {
 	pub fn with_id(id: String) -> TestLogger {
 		TestLogger {
 			level: Level::Trace,
+			#[cfg(feature = "std")]
 			id,
+			#[cfg(not(feature = "std"))]
+			_id: id,
 			lines: Mutex::new(HashMap::new())
 		}
 	}
@@ -462,15 +469,15 @@ impl Logger for TestLogger {
 	fn log(&self, record: &Record) {
 		*self.lines.lock().unwrap().entry((record.module_path.to_string(), format!("{}", record.args))).or_insert(0) += 1;
 		if record.level >= self.level {
+			#[cfg(feature = "std")]
 			println!("{:<5} {} [{} : {}, {}] {}", record.level.to_string(), self.id, record.module_path, record.file, record.line, record.args);
 		}
 	}
 }
 
 pub struct TestKeysInterface {
-	pub backing: keysinterface::KeysManager,
-	pub override_session_priv: Mutex<Option<[u8; 32]>>,
-	pub override_channel_id_priv: Mutex<Option<[u8; 32]>>,
+	pub backing: keysinterface::PhantomKeysManager,
+	pub override_random_bytes: Mutex<Option<[u8; 32]>>,
 	pub disable_revocation_policy_check: bool,
 	enforcement_states: Mutex<HashMap<[u8;32], Arc<Mutex<EnforcementState>>>>,
 	expectations: Mutex<Option<VecDeque<OnGetShutdownScriptpubkey>>>,
@@ -479,8 +486,12 @@ pub struct TestKeysInterface {
 impl keysinterface::KeysInterface for TestKeysInterface {
 	type Signer = EnforcingSigner;
 
-	fn get_node_secret(&self) -> SecretKey { self.backing.get_node_secret() }
-	fn get_inbound_payment_key_material(&self) -> keysinterface::KeyMaterial { self.backing.get_inbound_payment_key_material() }
+	fn get_node_secret(&self, recipient: Recipient) -> Result<SecretKey, ()> {
+		self.backing.get_node_secret(recipient)
+	}
+	fn get_inbound_payment_key_material(&self) -> keysinterface::KeyMaterial {
+		self.backing.get_inbound_payment_key_material()
+	}
 	fn get_destination_script(&self) -> Script { self.backing.get_destination_script() }
 
 	fn get_shutdown_scriptpubkey(&self) -> ShutdownScript {
@@ -500,16 +511,9 @@ impl keysinterface::KeysInterface for TestKeysInterface {
 	}
 
 	fn get_secure_random_bytes(&self) -> [u8; 32] {
-		let override_channel_id = self.override_channel_id_priv.lock().unwrap();
-		let override_session_key = self.override_session_priv.lock().unwrap();
-		if override_channel_id.is_some() && override_session_key.is_some() {
-			panic!("We don't know which override key to use!");
-		}
-		if let Some(key) = &*override_channel_id {
-			return *key;
-		}
-		if let Some(key) = &*override_session_key {
-			return *key;
+		let override_random_bytes = self.override_random_bytes.lock().unwrap();
+		if let Some(bytes) = &*override_random_bytes {
+			return *bytes;
 		}
 		self.backing.get_secure_random_bytes()
 	}
@@ -517,7 +521,7 @@ impl keysinterface::KeysInterface for TestKeysInterface {
 	fn read_chan_signer(&self, buffer: &[u8]) -> Result<Self::Signer, msgs::DecodeError> {
 		let mut reader = io::Cursor::new(buffer);
 
-		let inner: InMemorySigner = Readable::read(&mut reader)?;
+		let inner: InMemorySigner = ReadableArgs::read(&mut reader, self.get_node_secret(Recipient::Node).unwrap())?;
 		let state = self.make_enforcement_state_cell(inner.commitment_seed);
 
 		Ok(EnforcingSigner::new_with_revoked(
@@ -527,8 +531,8 @@ impl keysinterface::KeysInterface for TestKeysInterface {
 		))
 	}
 
-	fn sign_invoice(&self, invoice_preimage: Vec<u8>) -> Result<RecoverableSignature, ()> {
-		self.backing.sign_invoice(invoice_preimage)
+	fn sign_invoice(&self, hrp_bytes: &[u8], invoice_data: &[u5], recipient: Recipient) -> Result<RecoverableSignature, ()> {
+		self.backing.sign_invoice(hrp_bytes, invoice_data, recipient)
 	}
 }
 
@@ -536,9 +540,8 @@ impl TestKeysInterface {
 	pub fn new(seed: &[u8; 32], network: Network) -> Self {
 		let now = Duration::from_secs(genesis_block(network).header.time as u64);
 		Self {
-			backing: keysinterface::KeysManager::new(seed, now.as_secs(), now.subsec_nanos()),
-			override_session_priv: Mutex::new(None),
-			override_channel_id_priv: Mutex::new(None),
+			backing: keysinterface::PhantomKeysManager::new(seed, now.as_secs(), now.subsec_nanos(), seed),
+			override_random_bytes: Mutex::new(None),
 			disable_revocation_policy_check: false,
 			enforcement_states: Mutex::new(HashMap::new()),
 			expectations: Mutex::new(None),
@@ -571,9 +574,17 @@ impl TestKeysInterface {
 	}
 }
 
+pub(crate) fn panicking() -> bool {
+	#[cfg(feature = "std")]
+	let panicking = ::std::thread::panicking();
+	#[cfg(not(feature = "std"))]
+	let panicking = false;
+	return panicking;
+}
+
 impl Drop for TestKeysInterface {
 	fn drop(&mut self) {
-		if std::thread::panicking() {
+		if panicking() {
 			return;
 		}
 
@@ -665,7 +676,7 @@ impl chain::Filter for TestChainSource {
 
 impl Drop for TestChainSource {
 	fn drop(&mut self) {
-		if std::thread::panicking() {
+		if panicking() {
 			return;
 		}
 
@@ -713,4 +724,4 @@ impl core::fmt::Debug for OnRegisterOutput {
 }
 
 /// A scorer useful in testing, when the passage of time isn't a concern.
-pub type TestScorer = ScorerUsingTime<Eternity>;
+pub type TestScorer = FixedPenaltyScorer;
